# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""some init routines to do the documentation"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from IPython.display import Image
from IPython.display import display_pretty, display_html, display_jpeg, display_png, display_json, display_latex, display_svg
idfeditor = Image(filename='../docs/images/idf3.png')
material_lists = Image(filename='../docs/images/material_lists.png')
html_snippet1 = Image(filename='../docs/images/html1.png')
html_snippet2 = Image(filename='../docs/images/html2.png')
plantloop1 = Image(filename='../docs/images/hhh1.png')
plantloop2 = Image(filename='../docs/images/hhh_new.png')
